@@ -5,7 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-'''perform automatic newline conversion
+'''perform automatic newline conversion (DEPRECATED)
 
   Deprecation: The win32text extension requires each user to configure
   the extension again and again for each clone since the configuration
@@ -41,12 +41,22 @@ pushed or pulled::
   # or pretxnchangegroup.cr = python:hgext.win32text.forbidcr
 '''
 
-from mercurial.i18n import _
-from mercurial.node import short
-from mercurial import util
-import re
+from __future__ import absolute_import
 
-testedwith = 'internal'
+import re
+from mercurial.i18n import _
+from mercurial.node import (
+    short,
+)
+from mercurial import (
+    util,
+)
+
+# Note for extension authors: ONLY specify testedwith = 'ships-with-hg-core' for
+# extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
+# be specifying the version(s) of Mercurial they are tested with, or
+# leave the attribute unspecified.
+testedwith = 'ships-with-hg-core'
 
 # regexp for single LF without CR preceding.
 re_single_lf = re.compile('(^|[^\r])\n', re.MULTILINE)
@@ -58,7 +68,7 @@ def checknewline(s, newline, ui=None, repo=None, filename=None):
     # warn if already has 'newline' in repository.
     # it might cause unexpected eol conversion.
     # see issue 302:
-    #   http://mercurial.selenic.com/bts/issue302
+    #   https://bz.mercurial-scm.org/302
     if newline in s and ui and filename and repo:
         ui.warn(_('WARNING: %s already has %s line endings\n'
                   'and does not need EOL conversion by the win32text plugin.\n'
@@ -167,6 +177,7 @@ def reposetup(ui, repo):
         repo.adddatafilter(name, fn)
 
 def extsetup(ui):
+    # deprecated config: win32text.warn
     if ui.configbool('win32text', 'warn', True):
         ui.warn(_("win32text is deprecated: "
-                  "http://mercurial.selenic.com/wiki/Win32TextExtension\n"))
+                  "https://mercurial-scm.org/wiki/Win32TextExtension\n"))
